@@ -31,58 +31,39 @@
 #include <errno.h>
 #include <gdbus.h>
 #include <sys/socket.h>
+#include <inttypes.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/rfcomm.h>
+#include <gobex.h>
 
-#include <openobex/obex.h>
-
-#include "bluetooth.h"
 #include "obexd.h"
 #include "obex.h"
 #include "obex-priv.h"
-#include "dbus.h"
-#include "logging.h"
+#include "server.h"
+#include "manager.h"
+#include "log.h"
 #include "btio.h"
 #include "service.h"
 
-#define OPENOBEX_MANAGER_PATH		"/"
-#define OPENOBEX_MANAGER_INTERFACE	OPENOBEX_SERVICE ".Manager"
-#define ERROR_INTERFACE			OPENOBEX_SERVICE ".Error"
-#define TRANSFER_INTERFACE		OPENOBEX_SERVICE ".Transfer"
-#define SESSION_INTERFACE		OPENOBEX_SERVICE ".Session"
+#define OPENOBEX_MANAGER_PATH "/"
+#define OPENOBEX_MANAGER_INTERFACE OPENOBEX_SERVICE ".Manager"
+#define ERROR_INTERFACE OPENOBEX_SERVICE ".Error"
+#define TRANSFER_INTERFACE OPENOBEX_SERVICE ".Transfer"
+#define SESSION_INTERFACE OPENOBEX_SERVICE ".Session"
 
 #define TIMEOUT 60*1000 /* Timeout for user response (miliseconds) */
 
 struct agent {
-	gchar		*bus_name;
-	gchar		*path;
-	gboolean	auth_pending;
-	gchar		*new_name;
-	gchar		*new_folder;
-	guint		watch_id;
+	char *bus_name;
+	char *path;
+	gboolean auth_pending;
+	char *new_name;
+	char *new_folder;
+	unsigned int watch_id;
 };
 
 static struct agent *agent = NULL;
 
-struct pending_request {
-	DBusPendingCall *call;
-	struct server *server;
-	gchar *adapter_path;
-	char address[18];
-	guint watch;
-	GIOChannel *io;
-};
-
-struct adapter_any {
-	char *path;		/* Adapter ANY path */
-	GSList *servers;	/* List of servers to register records */
-};
-
 static DBusConnection *connection = NULL;
-static DBusConnection *system_conn = NULL;
-static struct adapter_any *any = NULL;
-static guint listener_id = 0;
 
 static void agent_free(struct agent *agent)
 {
@@ -201,7 +182,7 @@ static void dbus_message_iter_append_dict_entry(DBusMessageIter *dict,
 
 static void agent_disconnected(DBusConnection *conn, void *user_data)
 {
-	debug("Agent exited");
+	DBG("Agent exited");
 	agent_free(agent);
 	agent = NULL;
 }
@@ -209,7 +190,7 @@ static void agent_disconnected(DBusConnection *conn, void *user_data)
 static DBusMessage *register_agent(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	const gchar *path, *sender;
+	const char *path, *sender;
 
 	if (agent)
 		return agent_already_exists(msg);
@@ -227,7 +208,7 @@ static DBusMessage *register_agent(DBusConnection *conn,
 	agent->watch_id = g_dbus_add_disconnect_watch(conn, sender,
 					agent_disconnected, NULL, NULL);
 
-	debug("Agent registered");
+	DBG("Agent registered");
 
 	return dbus_message_new_method_return(msg);
 }
@@ -235,7 +216,7 @@ static DBusMessage *register_agent(DBusConnection *conn,
 static DBusMessage *unregister_agent(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	const gchar *path, *sender;
+	const char *path, *sender;
 
 	if (!agent)
 		return agent_does_not_exist(msg);
@@ -257,7 +238,7 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 	agent_free(agent);
 	agent = NULL;
 
-	debug("Agent unregistered");
+	DBG("Agent unregistered");
 
 	return dbus_message_new_method_return(msg);
 }
@@ -269,7 +250,7 @@ static char *target2str(const uint8_t *t)
 
 	return g_strdup_printf("%02X%02X%02X%02X-%02X%02X-%02X%02X-"
 				"%02X%02X-%02X%02X%02X%02X%02X%02X",
-				t[0], t[1], t[2], t[3], t[4], t[5], t[6],t[7],
+				t[0], t[1], t[2], t[3], t[4], t[5], t[6], t[7],
 				t[8], t[9], t[10], t[11], t[12], t[13], t[14], t[15]);
 }
 
@@ -280,7 +261,8 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 	DBusMessageIter dict;
-	gchar *uuid;
+	char *uuid;
+	const char *root;
 
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
@@ -299,8 +281,9 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	g_free(uuid);
 
 	/* Root folder */
+	root = obex_option_root_folder();
 	dbus_message_iter_append_dict_entry(&dict, "Root",
-					DBUS_TYPE_STRING, &os->server->folder);
+					DBUS_TYPE_STRING, &root);
 
 	/* FIXME: Added Remote Address or USB */
 
@@ -313,7 +296,7 @@ static DBusMessage *transfer_cancel(DBusConnection *connection,
 				DBusMessage *msg, void *user_data)
 {
 	struct obex_session *os = user_data;
-	const gchar *sender;
+	const char *sender;
 
 	if (!os)
 		return invalid_args(msg);
@@ -356,183 +339,15 @@ static GDBusMethodTable session_methods[] = {
 	{ }
 };
 
-static void add_record_reply(DBusPendingCall *call, gpointer user_data)
-{
-	struct server *server = user_data;
-	DBusMessage *reply = dbus_pending_call_steal_reply(call);
-	DBusError derr;
-	uint32_t handle;
-
-	dbus_error_init(&derr);
-	if (dbus_set_error_from_message(&derr, reply)) {
-		error("Replied with an error: %s, %s",
-				derr.name, derr.message);
-		dbus_error_free(&derr);
-		handle = 0;
-	} else {
-		struct obex_service_driver *driver;
-
-		dbus_message_get_args(reply, NULL,
-				DBUS_TYPE_UINT32, &handle,
-				DBUS_TYPE_INVALID);
-		server->handle = handle;
-
-		driver = (struct obex_service_driver *) server->drivers->data;
-
-		debug("Registered: %s, handle: 0x%x, folder: %s",
-				driver->name, handle, server->folder);
-	}
-
-	dbus_message_unref(reply);
-}
-
-static gint add_record(const gchar *path,
-		const gchar *xml, struct server *server)
-{
-	DBusMessage *msg;
-	DBusPendingCall *call;
-	gint ret = 0;
-
-	msg = dbus_message_new_method_call("org.bluez", path,
-					"org.bluez.Service", "AddRecord");
-
-	dbus_message_append_args(msg, DBUS_TYPE_STRING, &xml,
-			DBUS_TYPE_INVALID);
-
-	if (dbus_connection_send_with_reply(system_conn,
-				msg, &call, -1) == FALSE) {
-		ret = -1;
-		goto failed;
-	}
-
-	dbus_pending_call_set_notify(call, add_record_reply, server, NULL);
-	dbus_pending_call_unref(call);
-
-failed:
-	dbus_message_unref(msg);
-	return ret;
-}
-
-void register_record(struct server *server)
-{
-	struct obex_service_driver *driver;
-	gchar *xml;
-	gint ret;
-
-	if (system_conn == NULL)
-		return;
-
-	if (any->path == NULL) {
-		/* Adapter ANY is not available yet: Add record later */
-		any->servers = g_slist_append(any->servers, server);
-		return;
-	}
-
-	driver = (struct obex_service_driver *) server->drivers->data;
-	xml = g_markup_printf_escaped(driver->record, driver->channel,
-					driver->name);
-	ret = add_record(any->path, xml, server);
-	g_free(xml);
-}
-
-static void find_adapter_any_reply(DBusPendingCall *call, gpointer user_data)
-{
-	DBusMessage *reply = dbus_pending_call_steal_reply(call);
-	const char *path;
-	gchar *xml;
-	GSList *l;
-	DBusError derr;
-
-	dbus_error_init(&derr);
-	if (dbus_set_error_from_message(&derr, reply)) {
-		error("Replied with an error: %s, %s",
-				derr.name, derr.message);
-		dbus_error_free(&derr);
-		bluetooth_stop();
-		goto done;
-	}
-
-	dbus_message_get_args(reply, NULL,
-			DBUS_TYPE_OBJECT_PATH, &path,
-			DBUS_TYPE_INVALID);
-	any->path = g_strdup(path);
-
-	for (l = any->servers; l; l = l->next) {
-		struct server *server = l->data;
-		struct obex_service_driver *driver;
-
-		driver = (struct obex_service_driver *) server->drivers->data;
-		xml = g_markup_printf_escaped(driver->record, driver->channel,
-						driver->name);
-		add_record(path, xml, server);
-		g_free(xml);
-	}
-
-done:
-	g_slist_free(any->servers);
-	any->servers = NULL;
-
-	dbus_message_unref(reply);
-}
-
-static DBusPendingCall *find_adapter(const char *pattern,
-				DBusPendingCallNotifyFunction function,
-				gpointer user_data)
-{
-	DBusMessage *msg;
-	DBusPendingCall *call;
-
-	debug("FindAdapter(%s)", pattern);
-
-	msg = dbus_message_new_method_call("org.bluez", "/",
-					"org.bluez.Manager", "FindAdapter");
-	if (!msg)
-		return NULL;
-
-	dbus_message_append_args(msg, DBUS_TYPE_STRING, &pattern,
-			DBUS_TYPE_INVALID);
-
-	if (!dbus_connection_send_with_reply(system_conn, msg, &call, -1)) {
-		dbus_message_unref(msg);
-		return NULL;
-	}
-
-	dbus_pending_call_set_notify(call, function, user_data, NULL);
-
-	dbus_message_unref(msg);
-
-	return call;
-}
-
-static void name_acquired(DBusConnection *conn, void *user_data)
-{
-	DBusPendingCall *call;
-
-	call = find_adapter("any", find_adapter_any_reply, NULL);
-	if (call)
-		dbus_pending_call_unref(call);
-
-	bluetooth_start();
-}
-
-static void name_released(DBusConnection *conn, void *user_data)
-{
-	g_free(any->path);
-	any->path = NULL;
-	bluetooth_stop();
-}
-
 gboolean manager_init(void)
 {
 	DBusError err;
 
 	DBG("");
 
-	any = g_new0(struct adapter_any, 1);
-
 	dbus_error_init(&err);
 
-	connection = g_dbus_setup_private(DBUS_BUS_SESSION, OPENOBEX_SERVICE,
+	connection = g_dbus_setup_bus(DBUS_BUS_SESSION, OPENOBEX_SERVICE,
 									&err);
 	if (connection == NULL) {
 		if (dbus_error_is_set(&err) == TRUE) {
@@ -543,16 +358,6 @@ gboolean manager_init(void)
 		return FALSE;
 	}
 
-	system_conn = g_dbus_setup_bus(DBUS_BUS_SYSTEM, NULL, NULL);
-	if (system_conn == NULL) {
-		dbus_connection_unref(connection);
-		connection = NULL;
-		return FALSE;
-	}
-
-	listener_id = g_dbus_add_service_watch(system_conn, "org.bluez",
-				name_acquired, name_released, NULL, NULL);
-
 	return g_dbus_register_interface(connection, OPENOBEX_MANAGER_PATH,
 					OPENOBEX_MANAGER_INTERFACE,
 					manager_methods, manager_signals, NULL,
@@ -571,22 +376,12 @@ void manager_cleanup(void)
 	if (agent)
 		agent_free(agent);
 
-	g_dbus_remove_watch(system_conn, listener_id);
-
-	if (any) {
-		g_free(any->path);
-		g_free(any);
-	}
-
-	if (system_conn)
-		dbus_connection_unref(system_conn);
-
 	dbus_connection_unref(connection);
 }
 
 void manager_emit_transfer_started(struct obex_session *os)
 {
-	gchar *path = g_strdup_printf("/transfer%u", os->cid);
+	char *path = g_strdup_printf("/transfer%u", os->id);
 
 	g_dbus_emit_signal(connection, OPENOBEX_MANAGER_PATH,
 			OPENOBEX_MANAGER_INTERFACE, "TransferStarted",
@@ -596,9 +391,9 @@ void manager_emit_transfer_started(struct obex_session *os)
 	g_free(path);
 }
 
-static void emit_transfer_completed(guint32 id, gboolean success)
+static void emit_transfer_completed(struct obex_session *os, gboolean success)
 {
-	gchar *path = g_strdup_printf("/transfer%u", id);
+	char *path = g_strdup_printf("/transfer%u", os->id);
 
 	g_dbus_emit_signal(connection, OPENOBEX_MANAGER_PATH,
 			OPENOBEX_MANAGER_INTERFACE, "TransferCompleted",
@@ -609,9 +404,10 @@ static void emit_transfer_completed(guint32 id, gboolean success)
 	g_free(path);
 }
 
-static void emit_transfer_progress(guint32 id, guint32 total, guint32 transfered)
+static void emit_transfer_progress(struct obex_session *os, uint32_t total,
+							uint32_t transfered)
 {
-	gchar *path = g_strdup_printf("/transfer%u", id);
+	char *path = g_strdup_printf("/transfer%u", os->id);
 
 	g_dbus_emit_signal(connection, path,
 			TRANSFER_INTERFACE, "Progress",
@@ -624,7 +420,7 @@ static void emit_transfer_progress(guint32 id, guint32 total, guint32 transfered
 
 void manager_register_transfer(struct obex_session *os)
 {
-	gchar *path = g_strdup_printf("/transfer%u", os->cid);
+	char *path = g_strdup_printf("/transfer%u", os->id);
 
 	if (!g_dbus_register_interface(connection, path,
 				TRANSFER_INTERFACE,
@@ -640,11 +436,11 @@ void manager_register_transfer(struct obex_session *os)
 
 void manager_unregister_transfer(struct obex_session *os)
 {
-	gchar *path = g_strdup_printf("/transfer%u", os->cid);
+	char *path = g_strdup_printf("/transfer%u", os->id);
 
 	/* Got an error during a transfer. */
 	if (os->object)
-		emit_transfer_completed(os->cid, os->offset == os->size);
+		emit_transfer_completed(os, os->offset == os->size);
 
 	g_dbus_unregister_interface(connection, path,
 				TRANSFER_INTERFACE);
@@ -652,20 +448,23 @@ void manager_unregister_transfer(struct obex_session *os)
 	g_free(path);
 }
 
-static void agent_cancel()
+static void agent_cancel(void)
 {
 	DBusMessage *msg;
 
+	if (agent == NULL)
+		return;
+
 	msg = dbus_message_new_method_call(agent->bus_name, agent->path,
 					"org.openobex.Agent", "Cancel");
 
 	g_dbus_send_message(connection, msg);
 }
 
-static void agent_reply(DBusPendingCall *call, gpointer user_data)
+static void agent_reply(DBusPendingCall *call, void *user_data)
 {
 	DBusMessage *reply = dbus_pending_call_steal_reply(call);
-	const gchar *name;
+	const char *name;
 	DBusError derr;
 	gboolean *got_reply = user_data;
 
@@ -694,8 +493,8 @@ static void agent_reply(DBusPendingCall *call, gpointer user_data)
 				DBUS_TYPE_STRING, &name,
 				DBUS_TYPE_INVALID)) {
 		/* Splits folder and name */
-		const gchar *slash = strrchr(name, '/');
-		debug("Agent replied with %s", name);
+		const char *slash = strrchr(name, '/');
+		DBG("Agent replied with %s", name);
 		if (!slash) {
 			agent->new_name = g_strdup(name);
 			agent->new_folder = NULL;
@@ -708,29 +507,24 @@ static void agent_reply(DBusPendingCall *call, gpointer user_data)
 	dbus_message_unref(reply);
 }
 
-static gboolean auth_error(GIOChannel *io, GIOCondition cond,
-			gpointer user_data)
+static gboolean auth_error(GIOChannel *io, GIOCondition cond, void *user_data)
 {
 	agent->auth_pending = FALSE;
 
 	return FALSE;
 }
 
-int manager_request_authorization(struct obex_session *os, gint32 time,
-		gchar **new_folder, gchar **new_name)
+int manager_request_authorization(struct obex_session *os, int32_t time,
+					char **new_folder, char **new_name)
 {
 	DBusMessage *msg;
 	DBusPendingCall *call;
-	GIOChannel *io;
-	struct sockaddr_rc addr;
-	socklen_t addrlen;
-	gchar address[18];
-	const gchar *bda = address;
-	const gchar *filename = (os->name ? os->name : "");
-	const gchar *type = (os->type ? os->type : "");
-	gchar *path;
-	guint watch, fd;
+	const char *filename = os->name ? os->name : "";
+	const char *type = os->type ? os->type : "";
+	char *path, *address;
+	unsigned int watch;
 	gboolean got_reply;
+	int err;
 
 	if (!agent)
 		return -1;
@@ -741,23 +535,18 @@ int manager_request_authorization(struct obex_session *os, gint32 time,
 	if (!new_folder || !new_name)
 		return -EINVAL;
 
-	memset(&addr, 0, sizeof(addr));
-	addrlen = sizeof(addr);
+	err = obex_getpeername(os, &address);
+	if (err < 0)
+		return err;
 
-	fd = OBEX_GetFD(os->obex);
-	if (getpeername(fd, (struct sockaddr *) &addr, &addrlen) < 0)
-		return -errno;
-
-	ba2str(&addr.rc_bdaddr, address);
-
-	path = g_strdup_printf("/transfer%d", os->cid);
+	path = g_strdup_printf("/transfer%u", GPOINTER_TO_UINT(os));
 
 	msg = dbus_message_new_method_call(agent->bus_name, agent->path,
 					"org.openobex.Agent", "Authorize");
 
 	dbus_message_append_args(msg,
 			DBUS_TYPE_OBJECT_PATH, &path,
-			DBUS_TYPE_STRING, &bda,
+			DBUS_TYPE_STRING, &address,
 			DBUS_TYPE_STRING, &filename,
 			DBUS_TYPE_STRING, &type,
 			DBUS_TYPE_INT32, &os->size,
@@ -765,6 +554,7 @@ int manager_request_authorization(struct obex_session *os, gint32 time,
 			DBUS_TYPE_INVALID);
 
 	g_free(path);
+	g_free(address);
 
 	if (!dbus_connection_send_with_reply(connection,
 					msg, &call, TIMEOUT)) {
@@ -778,11 +568,9 @@ int manager_request_authorization(struct obex_session *os, gint32 time,
 	got_reply = FALSE;
 
 	/* Catches errors before authorization response comes */
-	io = g_io_channel_unix_new(fd);
-	watch = g_io_add_watch_full(io, G_PRIORITY_DEFAULT,
+	watch = g_io_add_watch_full(os->io, G_PRIORITY_DEFAULT,
 			G_IO_HUP | G_IO_ERR | G_IO_NVAL,
 			auth_error, NULL, NULL);
-	g_io_channel_unref(io);
 
 	dbus_pending_call_set_notify(call, agent_reply, &got_reply, NULL);
 
@@ -810,177 +598,9 @@ int manager_request_authorization(struct obex_session *os, gint32 time,
 	return 0;
 }
 
-static void service_cancel(struct pending_request *pending)
-{
-	DBusMessage *msg;
-
-	msg = dbus_message_new_method_call("org.bluez",
-					pending->adapter_path,
-					"org.bluez.Service",
-					"CancelAuthorization");
-
-	g_dbus_send_message(system_conn, msg);
-}
-
-static void pending_request_free(struct pending_request *pending)
-{
-	if (pending->call)
-		dbus_pending_call_unref(pending->call);
-	g_io_channel_unref(pending->io);
-	g_free(pending->adapter_path);
-	g_free(pending);
-}
-
-static void service_reply(DBusPendingCall *call, gpointer user_data)
-{
-	struct pending_request *pending = user_data;
-	GIOChannel *io = pending->io;
-	struct server *server = pending->server;
-	DBusMessage *reply = dbus_pending_call_steal_reply(call);
-	DBusError derr;
-	GError *err = NULL;
-
-	dbus_error_init(&derr);
-	if (dbus_set_error_from_message(&derr, reply)) {
-		error("RequestAuthorization error: %s, %s",
-				derr.name, derr.message);
-
-		if (dbus_error_has_name(&derr, DBUS_ERROR_NO_REPLY))
-			service_cancel(pending);
-
-		dbus_error_free(&derr);
-		g_io_channel_shutdown(io, TRUE, NULL);
-		goto done;
-	}
-
-	debug("RequestAuthorization succeeded");
-
-	if (!bt_io_accept(io, obex_connect_cb, server, NULL, &err)) {
-		error("%s", err->message);
-		g_error_free(err);
-		g_io_channel_shutdown(io, TRUE, NULL);
-	}
-
-done:
-	g_source_remove(pending->watch);
-	pending_request_free(pending);
-	dbus_message_unref(reply);
-}
-
-static gboolean service_error(GIOChannel *io, GIOCondition cond,
-			gpointer user_data)
-{
-	struct pending_request *pending = user_data;
-
-	service_cancel(pending);
-
-	dbus_pending_call_cancel(pending->call);
-
-	pending_request_free(pending);
-
-	return FALSE;
-}
-
-static void find_adapter_reply(DBusPendingCall *call, gpointer user_data)
-{
-	struct pending_request *pending = user_data;
-	DBusMessage *reply = dbus_pending_call_steal_reply(call);
-	DBusMessage *msg;
-	DBusPendingCall *pcall;
-	const char *path, *paddr = pending->address;
-	DBusError derr;
-
-	dbus_error_init(&derr);
-	if (dbus_set_error_from_message(&derr, reply)) {
-		error("Replied with an error: %s, %s",
-				derr.name, derr.message);
-		dbus_error_free(&derr);
-		goto failed;
-	}
-
-	dbus_message_get_args(reply, NULL,
-			DBUS_TYPE_OBJECT_PATH, &path,
-			DBUS_TYPE_INVALID);
-
-	debug("FindAdapter -> %s", path);
-	pending->adapter_path = g_strdup(path);
-	dbus_message_unref(reply);
-
-	msg = dbus_message_new_method_call("org.bluez", path,
-			"org.bluez.Service", "RequestAuthorization");
-
-	dbus_message_append_args(msg, DBUS_TYPE_STRING, &paddr,
-			DBUS_TYPE_UINT32, &pending->server->handle,
-			DBUS_TYPE_INVALID);
-
-	if (!dbus_connection_send_with_reply(system_conn,
-					msg, &pcall, TIMEOUT)) {
-		dbus_message_unref(msg);
-		goto failed;
-	}
-
-	dbus_message_unref(msg);
-
-	debug("RequestAuthorization(%s, %x)", paddr, pending->server->handle);
-
-	if (!dbus_pending_call_set_notify(pcall, service_reply, pending,
-								NULL)) {
-		dbus_pending_call_unref(pcall);
-		goto failed;
-	}
-
-	dbus_pending_call_unref(pending->call);
-	pending->call = pcall;
-
-	/* Catches errors before authorization response comes */
-	pending->watch = g_io_add_watch(pending->io,
-					G_IO_HUP | G_IO_ERR | G_IO_NVAL,
-					service_error, pending);
-
-	return;
-
-failed:
-	g_io_channel_shutdown(pending->io, TRUE, NULL);
-	pending_request_free(pending);
-}
-
-gint request_service_authorization(struct server *server, GIOChannel *io,
-					const char *address)
-{
-	struct pending_request *pending;
-	char source[18];
-	GError *err = NULL;
-
-	if (system_conn == NULL || any->path == NULL)
-		return -1;
-
-	bt_io_get(io, BT_IO_RFCOMM, &err,
-			BT_IO_OPT_SOURCE, source,
-			BT_IO_OPT_INVALID);
-	if (err) {
-		error("%s", err->message);
-		g_error_free(err);
-		return -EINVAL;
-	}
-
-	pending = g_new0(struct pending_request, 1);
-	pending->call = find_adapter(source, find_adapter_reply, pending);
-	if (!pending->call) {
-		g_free(pending);
-		return -ENOMEM;
-	}
-
-	pending->server = server;
-	pending->io = g_io_channel_ref(io);
-	memcpy(pending->address, address, sizeof(pending->address));
-
-
-	return 0;
-}
-
 void manager_register_session(struct obex_session *os)
 {
-	gchar *path = g_strdup_printf("/session%u", os->cid);
+	char *path = g_strdup_printf("/session%u", GPOINTER_TO_UINT(os));
 
 	if (!g_dbus_register_interface(connection, path,
 				SESSION_INTERFACE,
@@ -1001,7 +621,7 @@ void manager_register_session(struct obex_session *os)
 
 void manager_unregister_session(struct obex_session *os)
 {
-	gchar *path = g_strdup_printf("/session%u", os->cid);
+	char *path = g_strdup_printf("/session%u", GPOINTER_TO_UINT(os));
 
 	g_dbus_emit_signal(connection, OPENOBEX_MANAGER_PATH,
 			OPENOBEX_MANAGER_INTERFACE, "SessionRemoved",
@@ -1016,15 +636,16 @@ void manager_unregister_session(struct obex_session *os)
 
 void manager_emit_transfer_progress(struct obex_session *os)
 {
-	emit_transfer_progress(os->cid, os->size, os->offset);
+	emit_transfer_progress(os, os->size, os->offset);
 }
 
 void manager_emit_transfer_completed(struct obex_session *os)
 {
-	emit_transfer_completed(os->cid, !os->aborted);
+	if (os->object)
+		emit_transfer_completed(os, !os->aborted);
 }
 
-DBusConnection *obex_dbus_get_connection(void)
+DBusConnection *manager_dbus_get_connection(void)
 {
 	if (connection == NULL)
 		return NULL;
